from django import template
from django.template.loader import render_to_string, get_template
from django.template.base import token_kwargs
from django.template.defaultfilters import stringfilter
from django.template import engines
from django.template.context import Context
from django.utils.safestring import mark_safe
from django.urls import reverse
import re

register = template.Library()

# @template.defaultfilters.stringfilter
# @template.library.
@register.filter(name='render_ui')
def render_ui(component_name, props_str=""):
    props_dict = {}
    try:
        for item in props_str.split(";"):
            if item.strip():
                key, val = item.split(":", 1)
                props_dict[key.strip()] = val.strip()
    except Exception as e:
        props_dict["error"] = str(e)
    return render_to_string(f"palette/components/{component_name}.html", {"props": props_dict})

@register.tag(name="palette_ui")
def do_palette_ui(parser, token):
    bits = token.split_contents()
    tag_name = bits[0]

    if len(bits) < 2:
        raise template.TemplateSyntaxError(f"{tag_name} requires at least the template path.")

    template_path = parser.compile_filter(bits[1])
    context_vars = {}

    if len(bits) >= 3 and bits[2] == "with":
        context_vars = token_kwargs(bits[3:], parser, support_legacy=False)

    # Look ahead to determine if the tag is a block or inline
    token_list = list(parser.tokens)
    if token_list and token_list[0].contents == "endpalette_ui":
        # Inline tag (next token is end)
        parser.delete_first_token()
        return PaletteUI(template_path, context_vars, None)

    # Otherwise, parse as a block tag
    nodelist = parser.parse(('endpalette_ui',))
    parser.delete_first_token()  # remove 'endpalette_ui'
    return PaletteUI(template_path, context_vars, nodelist)


class PaletteUI(template.Node):
    def __init__(self, template_path, context_vars, nodelist=None):
        self.template_path = template_path
        self.context_vars = context_vars
        self.nodelist = nodelist

    def render_template_string(self, template_string, context):
        engine = engines['django']
        tmpl = engine.from_string(template_string)
        return tmpl.render(context.flatten())

    def render(self, context):
        template_path = self.template_path.resolve(context)
        template_obj = get_template(template_path)

        base_context = context.flatten()

        # Render and resolve variables
        extra_context = {}
        for key, val in self.context_vars.items():
            resolved = val.resolve(context)
            if isinstance(resolved, str) and ("{{" in resolved or "{%" in resolved):
                rendered = self.render_template_string(resolved, context)
                extra_context[key] = mark_safe(rendered)
            else:
                extra_context[key] = mark_safe(resolved)

        combined_context = {**base_context, **extra_context}
        rendered_component = template_obj.render(combined_context)

        # If no block content, just return the rendered component
        if not self.nodelist:
            return mark_safe(rendered_component)

        # Else render block with palette_ui.super
        context.push()
        context['palette_ui'] = {'super': mark_safe(rendered_component)}
        rendered_block = self.nodelist.render(context)
        context.pop()
        return mark_safe(rendered_block)


@register.simple_tag(takes_context=True, name="back_button")
def back_button(context):
    path = context['request'].path
    # Matches /admin/app/model/add/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/add/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/pk/change/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/\d+/change/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/$', path)
    if m:
        app = m.group(1)
        return f"/admin/{app}/"
    # Default: admin index
    return reverse('admin:index')





from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.admin.views.decorators import staff_member_required
from django.views.decorators.csrf import csrf_protect
from .models import AdminPage, Component
import json

@staff_member_required
def dashboard_view(request):
    return render(request, 'palette/dashboard.html', {
        "card_component": "card"
    })

@staff_member_required
def dynamic_admin_page(request, slug):
    page = get_object_or_404(AdminPage, slug=slug)
    try:
        layout = json.loads(page.layout_json)
    except json.JSONDecodeError:
        layout = []

    return render(request, 'palette/dynamic_page.html', {
        "page": page,
        "layout": layout
    })

@staff_member_required
@csrf_protect
def edit_admin_page(request, slug):
    page = get_object_or_404(AdminPage, slug=slug)
    components = Component.objects.all()

    if request.method == "POST":
        page.layout_json = request.POST.get("layout_json", "")
        page.save()
        return redirect("palette:edit-admin-page", slug=slug)

    return render(request, "palette/edit_page.html", {
        "page": page,
        "components": components
    })





from django.contrib import admin
from .site import PaletteAdminSite
from .models import AdminPage, Component
from django.contrib.auth.models import Group
from django.contrib.auth import get_user_model

User = get_user_model()

# admin.site.register(AdminPage)
# admin.site.register(Component)

palette_admin = PaletteAdminSite(name='palette_admin')
palette_admin.register(AdminPage)
palette_admin.register(Component)
palette_admin.register(User)
palette_admin.register(Group)



from django import template
from django.urls import reverse
import re

register = template.Library()

@register.simple_tag(takes_context=True)
def admin_back_url(context):
    path = context['request'].path
    # Matches /admin/app/model/add/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/add/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/pk/change/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/\d+/change/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/$', path)
    if m:
        app = m.group(1)
        return f"/admin/{app}/"
    # Default: admin index
    return reverse('admin:index')




# Generated by Django 5.2.4 on 2025-07-11 20:18

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Component',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('display_name', models.CharField(max_length=100)),
                ('default_props', models.JSONField(default=dict)),
                ('description', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='AdminPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('slug', models.SlugField(unique=True)),
                ('layout_json', models.TextField(help_text='JSON describing component layout')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.db import models
from django.contrib.auth import get_user_model

User = get_user_model()

class AdminPage(models.Model):
    title = models.CharField(max_length=200)
    slug = models.SlugField(unique=True)
    layout_json = models.TextField(help_text="JSON describing component layout")
    created_by = models.ForeignKey(User, on_delete=models.SET_NULL, null=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.title



class Component(models.Model):
    name = models.CharField(max_length=100, unique=True)
    display_name = models.CharField(max_length=100)
    default_props = models.JSONField(default=dict)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.display_name



from setuptools import setup, find_packages

setup(
    name="dj_palette",
    version="0.1.0",
    description="Build beautiful, component-based Django admin pages with drag-and-drop layouts.",
    author="Joel O. Tanko",
    author_email="7thogofe@email.com",
    url="https://github.com/ogofe/django-palette",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "Django>=2.2",
    ],
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.7",
)



from django.contrib.admin import AdminSite
from django.urls import path, re_path
from .views import dashboard_view, dynamic_admin_page, edit_admin_page
from django.template.response import TemplateResponse
from django.contrib.auth import logout, login
from django.shortcuts import redirect
from datetime import datetime
from django.apps import apps
from django.http import Http404
from django.utils.module_loading import import_string


PALETTE_SETTINGS = {
  'site_header': 'Daily Post',
  'index_title': 'Dashboard',
  'custom_urls': [],
  'custom_links': [
        {
            'label': "Support",
            'url_name': 'admin:support_chat', # a name from a valid url pattern,
            'left_icon': 'bi bi-support-chat', # or right_icon for positioning of the icon
        }
    ],

}




class PaletteAdminSite(AdminSite):
    # make the following variables dynamic from settings, e.g

    site_header = PALETTE_SETTINGS.get('site_header', "Palette Admin")
    site_title = PALETTE_SETTINGS.get('site_title', "Palette Admin")
    index_title = PALETTE_SETTINGS.get('index_title', "Palette Admin")
    # login_template = 'admin/login.html'
    # logout_template = 'admin/logout.html'

    def get_urls(self):
        custom_urls = [
            path('login/', self.login, name='login'),
            path('logout/', self.logout, name='logout'),
            path('password_change/', self.password_change, name='password_change'),
            path('password_change/done/', self.password_change_done, name='password_change_done'),
            # path('password_reset/', self.password_reset, name='password_reset'),
            # path('password_reset/done/', self.password_reset_done, name='password_reset_done'),
            # path('reset/<uidb64>/<token>/', self.password_reset_confirm, name='password_reset_confirm'),
            # path('reset/done/', self.password_reset_complete, name='password_reset_complete'),
            path('support_chat/', self.admin_view(self.support_chat_view), name='support_chat'),
            path('profile/', self.admin_view(self.profile_view), name='user-profile'),
            re_path(r'^(?P<app_label>\w+)/$', self.admin_view(self.app_index), name='app_list'),
        ]

        if PALETTE_SETTINGS.get('show_editor', True):
            custom_urls.extend([
                path('pages/', dashboard_view, name='custom-dashboard'),
                path('pages/edit/<slug:slug>/', edit_admin_page, name='edit-admin-page'),
                path('pages/<slug:slug>/', dynamic_admin_page, name='dynamic-admin-page'),
            ])
        if PALETTE_SETTINGS.get('custom_urls'):
            other_urls = PALETTE_SETTINGS.get('custom_urls', [])
            custom_urls.extend(other_urls)
        return custom_urls + super().get_urls()

    def logout(self, request, **kwargs):
        logout(request)
        return redirect('admin:login') # Redirect to the login page after logout

    def get_app_list(self, request):
        app_dict = self._build_app_dict(request)

        # Inject model manager as model.objects for template use
        for app_label, app in app_dict.items():
            for model_dict in app['models']:
                model_class = self._registry[model_dict['model']].model
                model_dict['objects'] = model_class._default_manager  # or .objects

        app_list = sorted(app_dict.values(), key=lambda x: x['name'].lower())
        for app in app_list:
            app['models'].sort(key=lambda x: x['name'])
        return app_list


    def each_context(self, request):
        context = super().each_context(request)
        context['custom_links'] = PALETTE_SETTINGS.get('custom_links', [])
        return context
    
    # Dashboard index view
    def index(self, request, extra_context=None, **kwargs):
        context = self.each_context(request)
        
        now = datetime.now().time()
        greeting = 'Morning'
        if 12 < now.hour <= 16:greeting='Afternoon'
        if now.hour > 16:greeting='Evening'

        context['greeting'] = greeting
        context['user'] = request.user
        return TemplateResponse(request, "admin/index.html", context)

    def app_index(self, request, app_label, extra_context=None):
        app_config = apps.get_app_config(app_label)
        if not app_config:
            raise Http404("App not found")

        context = {
            **self.each_context(request),
            "title": app_config.verbose_name,
            "app_label": app_label,
            "app_list": [app for app in self.get_app_list(request) if app['app_label'] == app_label],
        }
        print("App List:", context['app_list'])
        context.update(extra_context or {})
        return TemplateResponse(request, "admin/app_index.html", context)


    def profile_view(self, request):
        """
        Custom view to handle profile viewing.
        """
        context = self.each_context(request)
        context['title'] = "User Profile"
        context['user'] = request.user
        return TemplateResponse(request, "admin/profile.html", context)

    def support_chat_view(self, request):
        """
        Custom view to handle profile viewing.
        """
        context = self.each_context(request)
        context['title'] = "User Profile"
        context['user'] = request.user
        return TemplateResponse(request, "admin/profile.html", context)
    
